import requests

from chatbot.models.facebook_message import FacebookMessage


class FacebookUtils(object):

    def __init__(self, fbaccesstoken):
        self.fbaccesstoken = fbaccesstoken

    def generate_response(self, response):
        if response.response_type == 'text':
            return {"text": response.message}
        elif response.response_type == 'image':
            return {"attachment": {"type": "image", "payload": {"url": response.message}}}
        elif response.response_type == 'image_reusable':
            return {"attachment": {"type": "image", "payload": {"attachment_id": response.message}}}
        elif response.response_type == 'video':
            return {"attachment": {"type": "video", "payload": {"url": response.message}}}
        elif response.response_type == 'audio':
            return {"attachment": {"type": "audio", "payload": {"url": response.message}}}
        elif response.response_type == 'file':
            return {"attachment": {"type": "file", "payload": {"url": response.message}}}
        elif response.response_type == 'text_buttons':
            buttons = []
            for button in response.message['buttons']:
                if button['type'] == 'web_url':
                    b = {"type": button['type'], "title": button['title'], "url": button['url'], "webview_height_ratio": button['webview_height_ratio']}
                    buttons.append(b)
                else:
                    b = {"content_type": button['type'], "title": button['title'], "payload": button['payload']}
                    buttons.append(b)
            data = {"text": response.message['text'], "quick_replies": buttons}
            return data
        elif response.response_type == 'horizontal_list' or response.response_type == 'horizontal_list_square':
            if response.response_type == 'horizontal_list_square':
                image_type = 'square'
            else:
                image_type = 'horizontal'
            elements = []
            for element in response.message:
                e = dict()
                e['title'] = element['title']
                if element.get('subtitle'):
                    e["subtitle"] = element['subtitle']
                if element.get('image_url'):
                    e["image_url"] = element['image_url']
                # if element.get('attachment_id'):
                #     e["attachment_id"] = element['attachment_id']
                if element.get('default_action'):
                    e["default_action"] = element['default_action']
                if element.get('buttons'):
                    buttons = []
                    for button in element['buttons']:
                        buttons.append(button)
                    e["buttons"] = buttons
                elements.append(e)
            data = {
                "attachment": {
                    "type": "template",
                    "payload": {
                        "template_type": "generic",
                        "image_aspect_ratio": image_type,
                        "elements": elements
                    }
                }
            }
            return data
        elif response.response_type == 'vertical_list_cover' or response.response_type == 'vertical_list':
            elements = []
            for element in response.message:
                e = dict()
                e['title'] = element['title']
                if element.get('subtitle'):
                    e["subtitle"] = element['subtitle']
                if element.get('image_url'):
                    e["image_url"] = element['image_url']
                if element.get('default_action'):
                    e["default_action"] = element['default_action']
                if element.get('buttons'):
                    buttons = []
                    for button in element['buttons']:
                        buttons.append(button)
                    e["buttons"] = buttons
                elements.append(e)
            if response.response_type == 'vertical_list':
                data = {
                    "attachment": {
                        "type": "template",
                        "payload": {
                            "template_type": "list",
                            "top_element_style": "compact",
                            "elements": elements
                        }
                    }
                }
            else:
                data = {
                    "attachment": {
                        "type": "template",
                        "payload": {
                            "template_type": "list",
                            "elements": elements
                        }
                    }
                }
            return data

    def parse_fb_message(self, data):
        """Parse Facebook message and return custom message object."""
        if data['object'] == 'page':
            for messagings in (entry['messaging'] for entry in data['entry']):
                for messaging in messagings:
                    sender = messaging['sender']['id']
                    message_type = None
                    message = None
                    image = None
                    location = None
                    if messaging.get('message'):
                        if messaging['message'].get('text'):
                            message = messaging['message']['text']
                            if messaging['message'].get('quick_reply'):
                                message_type = messaging['message']['quick_reply'].get('payload')
                                return FacebookMessage(sender, message, message_type, image, location)
                            else:
                                message_type = "USER_TEXT"
                        elif messaging['message'].get('attachments'):
                            for attachment in messaging['message']['attachments']:
                                if attachment['type'] == 'image':
                                    image = attachment["payload"]['url']
                                    message_type = "USER_IMAGE"
                                elif attachment['type'] == 'location':
                                    location = {"type": "location", "data": {
                                        "coordinates": attachment["payload"]['coordinates']}}
                                    message_type = "USER_LOCATION"
                    elif messaging.get('delivery'):
                        pass
                    elif messaging.get('option'):
                        pass
                    elif messaging.get('postback'):
                        message_type = messaging['postback'].get('payload')
                    return FacebookMessage(sender, message, message_type, image, location)

    def typing_on(self, user_id):
        data = {
            "recipient": {"id": user_id},
            "sender_action": "typing_on"
        }
        requests.post("https://graph.facebook.com/v2.6/me/messages?access_token=" + self.fbaccesstoken,
                             json=data)

    def typing_off(self, user_id):
        data = {
            "recipient": {"id": user_id},
            "sender_action": "typing_off"
        }
        requests.post("https://graph.facebook.com/v2.6/me/messages?access_token=" + self.fbaccesstoken,
                             json=data)

    def whitelist(self, whitelist):
        resp = requests.post(
            "https://graph.facebook.com/v2.6/me/thread_settings?access_token=" + self.fbaccesstoken,
            json=whitelist)
        print(resp.content)

    def quick_answer(self, sender, data):
        response = {
            "recipient": {"id": sender},
            "message": data
        }
        resp = requests.post("https://graph.facebook.com/v2.6/me/messages?access_token=" + self.fbaccesstoken,
                             json=response)
        self.typing_off(sender)
        print(resp.content)

    def seen(self, user_id):
        data = {
            "recipient": {"id": user_id},
            "sender_action": "mark_seen"
        }
        resp = requests.post("https://graph.facebook.com/v2.6/me/messages?access_token=" + self.fbaccesstoken,
                             json=data)
        print(resp.content)

    def get_started_button(self, getstarted):
        resp = requests.post(
            "https://graph.facebook.com/v2.6/me/thread_settings?access_token=" + self.fbaccesstoken, json=getstarted)
        print(resp.content)

    def setup_persistant_menu(self, menu):
        resp = requests.post(
            "https://graph.facebook.com/v2.6/me/thread_settings?access_token=" + self.fbaccesstoken, json=menu)
        print(resp.content)

    def upload_image(self, image):
        response = {
            "message": {
                "attachment": {
                    "type": "image",
                    "payload": {
                        "url": image,
                        "is_reusable": True,
                    }
                }
            }
        }
        resp = requests.post("https://graph.facebook.com/v2.6/me/message_attachments?access_token=" + self.fbaccesstoken,
                             json=response)
        print(resp.content)
        return resp



from abc import ABCMeta, abstractmethod


class UserInputMessage(object):

    __metaclass__ = ABCMeta

    __from_user__ = True

    def __init__(self, sender, message, message_type, image, location):
        self.sender = sender
        self.message = message
        self.message_type = message_type
        self.image = image
        self.location = location

    @abstractmethod
    def message_service(self):
        """"Return a string representing the service where message came from."""



class ResponseMessage(object):

    __from_user__ = False
    __message_service__ = 'service'

    def __init__(self, sender, service, message, message_type, response_type):
        self.sender = sender
        self.service = service
        self.message = message
        self.message_type = message_type
        self.response_type = response_type



from chatterbot import ChatBot
from pymongo import MongoClient


class ChatterbotUtils(object):
    def __init__(self, chatterbotcfg):
        self.chatterbotcfg = chatterbotcfg
        self.chatterbot = ChatBot(
            self.chatterbotcfg['name'],
            trainer="chatterbot.trainers.ChatterBotCorpusTrainer",
            input_adapter="chatterbot.input.VariableInputTypeAdapter",
            output_adapter="chatterbot.output.OutputFormatAdapter",
            storage_adapter="chatterbot.storage.MongoDatabaseAdapter",
            logic_adapters=[
                {
                    'import_path': 'chatterbot.logic.BestMatch'
                },
                {
                    'import_path': 'chatterbot.logic.LowConfidenceAdapter',
                    'threshold': 0.65,
                    'default_response': self.chatterbotcfg['default_response']
                }
            ],
            database=self.chatterbotcfg['database'],
            read_only=self.chatterbotcfg['readonly'],
        )
        self.chatterbot.train("corpus")
        # self.drop_chatterbot_database()

    def drop_chatterbot_database(self):
        """Drop chatterbot database and train new one."""
        client = MongoClient(self.chatterbotcfg['host'], self.chatterbotcfg['port'])
        client.drop_database(self.chatterbotcfg['name'])
        self.chatterbot.train("corpus")

    def generate_chatterbot_response(self, message):
        """Generate chatterbot response"""
        return self.chatterbot.get_response(message)



from abc import ABCMeta, abstractmethod
from time import sleep

import simplejson

from chatbot.models.response_message import ResponseMessage
from chatbot.utils.chatterbot_utils import ChatterbotUtils
from chatbot.utils.database_utils import DatabaseUtils
from chatbot.utils.facebook_utils import FacebookUtils


class ContentGenerator(object):

    __metaclass__ = ABCMeta

    def __init__(self, config):
        self.db_utils = DatabaseUtils(config.database, config.facebook['access_token'])
        self.chatterbot_utils = ChatterbotUtils(config.chatterbot)
        self.fb_utils = FacebookUtils(config.facebook['access_token'])
        # self.mobile_utils = MobileUtils(constants)
        # self.web_utils = WebUtils(constants)
        # self.kik_utils = KikUtils(constants)

    @abstractmethod
    def run_method(self, name, data):
        """Run method that is required"""
        pass

    @abstractmethod
    def parse_input(self, message):
        """Parse user input to check if it matches expected input."""
        pass

    @abstractmethod
    def text_response(self, response):
        pass

    def handle_input(self, message):
        """Decide what kind of response is needed."""
        if message.message_type == 'USER_TEXT':
            if self.db_utils.is_input_expected(message.sender):
                response = ResponseMessage(message.sender, message.message_service(), message.message, None, None)
                self.run_method("parse_input", response)
            else:
                val = self.chatterbot_utils.generate_chatterbot_response(message.message)
                res = dict()
                try:
                    res = simplejson.loads(str(val))
                except simplejson.decoder.JSONDecodeError:
                    res['data'] = str(val)
                    res['type'] = "text_response"
                response = ResponseMessage(message.sender, message.message_service(), res['data'], res['type'], None)
                self.run_method(res['type'], response)
        else:
            response = ResponseMessage(message.sender, message.message_service(), message, None, None)
            self.run_method(message.message_type, response)

    def sleep_and_typing(self, response, time):
        if response.service == 'facebook':
            self.fb_utils.typing_on(response.sender)
            sleep(time)

    def send_response(self, response, time):
        print("sender = {0}".format(response.sender))
        if response.service == 'facebook':
            self.fb_utils.typing_on(response.sender)
            sleep(time)
            data = self.fb_utils.generate_response(response)
            self.send_fb_response(response.sender, data)
        # if response.service == 'mobile':
        #     data = self.mobile_utils.generate_response(response)
        #     self.send_mobile_response(response.sender, data)
        # if response.service == 'web':
        #     data = self.web_utils.generate_response(response)
        #     self.send_web_response(response.sender, data)
        # if response.service == 'kik':
        #     data = ""
        #     self.send_kik_response(response.sender, data)

    def send_fb_response(self, sender, data):
        self.fb_utils.quick_answer(sender, data)

    # def send_mobile_response(self, sender, data):
    #     self.mobile_utils.send(sender, data)
    #
    # def send_web_response(self, sender, data):
    #     self.web_utils.send(sender, data)
    #
    # def send_kik_response(self, sender, data):
    #     self.kik_utils.send(sender, data)

    def create_button(self, type, text, payload, url=None, wv_height=None, extensions=None, fallback=None):
        button = dict()
        if type == 'text' or type == 'postback':
            button['type'] = type
            button['title'] = text
            button['payload'] = payload
        elif type == 'web_url':
            button['type'] = type
            button['title'] = text
            button['url'] = url
            if wv_height:
                button['webview_height_ratio'] = wv_height
            if extensions:
                button['messenger_extensions'] = extensions
            if fallback:
                button['fallback_url'] = fallback
        elif type == 'location':
            button['type'] = 'location'
            button['title'] = text
            button['payload'] = payload
        return button

    def create_default_action(self, type, title, url, extensions=None, wv_height=None, fallback=None):
        action = dict()
        action['type'] = type
        action['title'] = title
        action['url'] = url
        if wv_height:
            action['webview_height_ratio'] = wv_height
        if extensions:
            action['messenger_extensions'] = extensions
        if fallback:
            action['fallback_url'] = fallback
        return action

    def create_element(self, title, subtitle=None, image_url=None, attachment=None, buttons=None, default_action=None):
        message = dict()
        message['title'] = title
        if subtitle:
            message['subtitle'] = subtitle
        if attachment:
            print("attachment_id = " + attachment)
            message['attachment_id'] = attachment
        if image_url:
            print("image url = " + image_url)
            message['image_url'] = image_url
        if buttons:
            message['buttons'] = buttons
        if default_action:
            message['default_action'] = default_action
        return message




from setuptools import setup

setup(name='martian_chatbot',
      version='0.0.4',
      description='Chatbot creation module',
      url='https://jzitkovic@gitlab.martianandmachine.com/service/martian-chatbot.git',
      author='Josip Zitkovic',
      author_email='josip@martian.agency',
      license='MIT',
      packages=['chatbot', 'chatbot.utils', 'chatbot.models'],
      install_requires=[
          'pymongo',
          'ChatterBot',
          'requests',
          'simplejson'
      ],
      zip_safe=False)



import uuid

import requests
import simplejson
from bson import ObjectId
from pymongo import MongoClient


class DatabaseUtils(object):
    def __init__(self, dbconfig, fbaccesstoken):
        self.dbconfig = dbconfig
        self.fbaccesstoken = fbaccesstoken
        client = MongoClient(dbconfig['host'], dbconfig['port'])
        # client.drop_database(constants.DATABASE_NAME)
        self.db = client[dbconfig['name']]

    def get_user(self, sender):
        """Returns user object from database."""
        user = self.db.User
        current_sender = None
        current_user = user.find({
            'user_id': str(sender)})
        for document in current_user:
            current_sender = document
        if current_sender is None:
            if sender.isdigit():
                current_sender = self.generate_new_fb_user(sender)
            else:
                current_sender = self.generate_new_user()
        return current_sender

    def generate_new_fb_user(self, sender):
        """Generates new Facebook user in database."""
        user = self.db.User
        current_sender = None
        try:
            resp = requests.get(
                'https://graph.facebook.com/v2.6/' + sender + '?access_token=' + self.fbaccesstoken)
            fb_user = simplejson.loads(str(resp.text))
            current_sender = dict()
            current_sender['_id'] = ObjectId()
            current_sender["is_input_expected"] = False
            current_sender["expected_input"] = ""
            current_sender['user_id'] = sender
            if fb_user.get('first_name'):
                current_sender['first_name'] = fb_user['first_name']
            if fb_user.get('last_name'):
                current_sender['last_name'] = fb_user['last_name']
            if fb_user.get('gender'):
                current_sender['gender'] = fb_user['gender']
            if fb_user.get('locale'):
                current_sender['locale'] = fb_user['locale']
            user.insert(current_sender)
            print('Created new user: {0}'.format(current_sender['user_id']))
        except:
            print('Error creating new user')
        return current_sender

    def generate_new_user(self):
        """Generates new user in database."""
        user = self.db.User
        user_id = uuid.uuid4().urn[9:]
        print('generated user = {0}'.format(user_id))
        current_sender = None
        current_user = user.find({
            'user_id': str(user_id)})
        for document in current_user:
            current_sender = document
        if current_sender is None:
            try:
                current_sender = self.constants.USER_OBJECT
                current_sender['user_id'] = user_id
                current_sender["is_input_expected"] = False
                current_sender["expected_input"] = ""
                print('Created new user: {0}'.format(user.insert_one(current_sender).inserted_id))
            except:
                print('Error creating new user')
        return current_sender

    def update_user(self, sender, data_for_update):
        """Updates user object in database."""
        self.db.User.update_one({
            'user_id': sender
        }, {
            '$set': data_for_update
        }, upsert=False)

    def is_input_expected(self, sender):
        """Checks if input is expected."""
        user = self.db.User
        current_user = user.find({
            "user_id": str(sender)})
        is_expected = False
        try:
            for document in current_user:
                is_expected = document["is_input_expected"]
        except:
            return False
        return is_expected

    def get_expected_input_type(self, sender):
        """Returns expected input type."""
        user = self.db.User
        current_user = user.find({
            "user_id": str(sender)})
        expected_input = ""
        for document in current_user:
            expected_input = document["expected_input"]
        return expected_input